import { css, jsx } from '@emotion/core';
import { Sampler, Frequency, now } from 'tone';
import { useState, useEffect, useCallback, useLayoutEffect as useLayoutEffect$1, useMemo, useRef, Fragment } from 'react';
import range from 'lodash.range';
import { midi, get, fromMidiSharps } from '@tonaljs/note';
import classNames from 'classnames';
import color from 'color';

function set(array, pos, item) {
    var newArray = array.slice(0);
    newArray[pos] = item;
    return newArray;
}

var makeTuning = function (text) {
    return text
        .split(' ')
        .map(function (name) { var _a; return (_a = midi(name)) !== null && _a !== void 0 ? _a : 0; })
        .reverse();
};
var tunings = {
    standard: makeTuning('E2 A2 D3 G3 B3 E4'),
    rondeña: makeTuning('D2 A2 D3 F#3 B3 E4'),
    dadgad: makeTuning('D2 A2 D3 G3 A3 D4'),
    ukelele: makeTuning('G4 C4 E4 A4'),
    lute: makeTuning('E2 A2 D3 F#3 B3 E4')
};

function useSound(samples, fretting, tuning, muted) {
    if (tuning === void 0) { tuning = tunings.standard; }
    var _a = useState(false), loaded = _a[0], setLoaded = _a[1];
    var _b = useState(), synth = _b[0], setSynth = _b[1];
    var _c = useState(tuning.map(function () { return false; })), playing = _c[0], setPlaying = _c[1];
    useEffect(function () {
        if (!muted) {
            var synth_1 = new Sampler(samples, function () { return setLoaded(true); }).toDestination();
            setSynth(synth_1);
            return function () {
                synth_1.dispose();
            };
        }
    }, [muted]);
    var play = useCallback(function (string, when) {
        var _a;
        if (when === void 0) { when = 0; }
        var fret = (_a = fretting[string]) !== null && _a !== void 0 ? _a : 0;
        if (loaded && !muted && synth && fret >= 0) {
            setTimeout(function () { return setPlaying(function (playing) { return set(playing, string, true); }); }, 0);
            setTimeout(function () { return setPlaying(function (playing) { return set(playing, string, false); }); }, 3000);
            synth.triggerAttackRelease(Frequency(tuning[string] + fret, 'midi').toFrequency(), 4, now() + when);
        }
    }, [loaded, muted, synth, fretting.toString(), tuning.toString()]);
    var strum = useCallback(function (up) {
        range(tuning.length).forEach(function (i) {
            var string = !up ? tuning.length - i - 1 : i;
            play(string, 0.05 * i);
        });
    }, [tuning.toString(), play]);
    return { play: play, strum: strum, playing: playing };
}

var useLayoutEffect = typeof window !== 'undefined' ? useLayoutEffect$1 : useEffect;

var makeTheme = function (theme) { return theme; };
var spanishTheme = makeTheme({
    color: '#333333',
    nut: { color: '#fffacd' },
    fret: {
        color: '#9e6429',
        separator: { color: '#daa520', shadow: true },
        counter: { color: '#606c76' }
    },
    string: { color: function () { return '#eeeeee'; } },
    finger: { text: { color: '#606c76' }, color: 'white' }
});

var sw = function (theme) { return (theme.fret.separator.width === 'md' ? 0.7 : 0.3); };
var height = 3.5 / 1.5 + "em";
var getStyles = (function (theme) {
    return css({
        fontFamily: "'Roboto', 'Helvetica Neue', 'Helvetica', 'Arial', sans-serif",
        position: 'relative',
        boxSizing: 'border-box',
        maxWidth: '100%',
        overflowY: 'auto',
        whiteSpace: 'nowrap',
        margin: 0,
        padding: 0,
        listStyle: 'none',
        borderTopColor: theme.color,
        borderTopStyle: 'solid',
        borderTopWidth: '0.5em',
        '&.lefty': {
            direction: 'rtl',
            '.marker': {
                transform: 'scale(-1, 1)'
            }
        },
        '*, *::before, *::after': {
            boxSizing: 'border-box'
        },
        '.frets': {
            display: 'flex',
            position: 'relative',
            '.fret': {
                width: '10em',
                flexShrink: 0,
                position: 'relative',
                '&.nut,&.mute': {
                    width: '7em'
                },
                '&.mute': {
                    position: 'absolute',
                    top: 0,
                    left: 0
                }
            }
        },
        '.frame': {
            height: '2em',
            '.counter': {
                fontWeight: 'bold',
                color: theme.fret.counter.color
            },
            '.fret': {
                display: 'flex',
                alignItems: 'center',
                justifyContent: 'center',
                background: theme.color
            }
        },
        '.strings': {
            display: 'flex',
            flexDirection: 'column',
            position: 'relative',
            '.string': {
                '.fret': {
                    display: 'flex',
                    alignItems: 'center',
                    '.actual-string': {
                        transition: 'opacity ease-in-out 0.1s',
                        content: '""',
                        width: '100%',
                        height: '0.65em',
                        position: 'absolute',
                        left: '0'
                    },
                    label: {
                        fontSize: '1em',
                        width: '100%',
                        height: '3.33125em',
                        display: 'flex',
                        alignItems: 'center',
                        justifyContent: 'center',
                        margin: 0
                    },
                    '&:hover input:not(:disabled):not(:checked) ~ .finger': {
                        opacity: 0.5
                    },
                    'input:not(:disabled)': {
                        height: '100%',
                        width: '100%'
                    },
                    'input:not(:disabled),input:not(:disabled) ~ .finger': {
                        cursor: 'pointer'
                    },
                    input: {
                        position: 'absolute',
                        margin: 0,
                        opacity: 0,
                        '&:checked ~ .finger': {
                            opacity: 1
                        },
                        '&:focus:not(:disabled) ~ .finger': {
                            boxShadow: '0 0 0 0.2em rgba(66, 153, 225, 0.5)'
                        }
                    },
                    '&.mute': {
                        'input:checked ~ .finger': {
                            opacity: 0
                        },
                        'input:focus ~ .finger,&:hover input:checked ~ .finger': {
                            opacity: 0.5
                        }
                    }
                }
            }
        },
        '.fretboard': {
            position: 'absolute',
            left: '0',
            right: '0',
            top: '0',
            bottom: '0',
            '.fret': {
                background: 'red',
                display: 'flex',
                flexDirection: 'row-reverse',
                alignItems: 'center',
                justifyContent: 'end',
                backgroundColor: theme.fret.color,
                position: 'relative',
                '&.nut': {
                    backgroundColor: theme.nut.color
                },
                '&:not(.nut)::before': {
                    content: '""',
                    position: 'absolute',
                    top: '0',
                    bottom: '0',
                    width: sw(theme) + "em",
                    backgroundColor: theme.fret.separator.color,
                    borderRight: theme.fret.separator.shadow
                        ? "solid " + sw(theme) / 2 + "em " + color(theme.fret.separator.color).darken(0.1)
                        : 0,
                    borderRadius: theme.fret.separator.radius ? '3px' : 0,
                    display: 'inline-block'
                },
                '.marker': {
                    position: 'absolute',
                    left: '0',
                    right: '0',
                    top: '0',
                    bottom: '0',
                    display: 'flex',
                    alignItems: 'center',
                    justifyContent: 'center'
                }
            }
        },
        '.finger': {
            color: theme.finger.text.color,
            transition: 'opacity ease-in-out 0.1s',
            background: theme.finger.color,
            width: '5em',
            padding: '0',
            height: height,
            borderRadius: '20px',
            borderBottom: "solid 0.2em " + color(theme.finger.color).darken(0.35),
            boxShadow: '0 1px 2px rgba(0, 0, 0, 0.16), 0 1px 2px rgba(0, 0, 0, 0.23)',
            lineHeight: height,
            textAlign: 'center',
            fontWeight: 'bold',
            opacity: 0,
            display: 'inline-flex',
            alignItems: 'center',
            justifyContent: 'center',
            position: 'relative'
        },
        '.sr-only': {
            position: 'absolute',
            width: '1px',
            height: '1px',
            padding: 0,
            margin: '-1px',
            overflow: 'hidden',
            clip: 'rect(0, 0, 0, 0)',
            whiteSpace: 'nowrap',
            borderWidth: 0
        }
    });
});

/** @jsx jsx */
function getRenderFingerSpn(tuning) {
    return function (string, fret) {
        var _a = get(fromMidiSharps(tuning[string] + fret)), letter = _a.letter, acc = _a.acc, oct = _a.oct;
        return (jsx("span", null,
            letter,
            acc === '#' ? '♯' : acc === 'b' ? '♭' : '',
            jsx("sub", { "aria-label": "octave " + oct }, oct)));
    };
}
function getRenderFingerRelative(tuning, root) {
    var mod = function (n, m) { return (m + (n % m)) % m; };
    return function (string, fret) { return (jsx(Fragment, null, ['1', '2m', '2', '3m', '3', '4', '5dim', '5', '5aug', '6', '7m', '7'][mod(tuning[string] + fret - root, 12)])); };
}
function Frets(props) {
    var _a;
    var _b = props.frets, from = _b.from, amount = _b.amount;
    return (jsx("div", { className: classNames(props.className, 'frets'), onMouseEnter: props.onMouseEnter },
        props.currentFret !== undefined && (jsx("div", { className: "fret mute", style: { zIndex: props.currentFret === -1 ? 1 : undefined } }, (_a = props.children) === null || _a === void 0 ? void 0 : _a.call(props, -1))),
        range(from, from + amount + 1).map(function (fret) {
            var _a;
            return (jsx("div", { className: classNames('fret', { nut: fret === 0 }), key: fret }, (_a = props.children) === null || _a === void 0 ? void 0 : _a.call(props, fret)));
        })));
}
function Guitar(props) {
    var _a = props.strings, strings = _a === void 0 ? [] : _a, _b = props.frets, frets = _b === void 0 ? { from: 0, amount: 22 } : _b, _c = props.lefty, lefty = _c === void 0 ? false : _c, _d = props.center, center = _d === void 0 ? false : _d, renderFinger = props.renderFinger, _e = props.theme, theme = _e === void 0 ? spanishTheme : _e, playOnHover = props.playOnHover;
    var styles = useMemo(function () { return getStyles(theme); }, [theme]);
    var ref = useRef(null);
    var releaseString = function (string) {
        var _a, _b, _c, _d;
        var newFret = strings[string] === 0 ? -1 : 0;
        (_c = (_b = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.querySelector) === null || _b === void 0 ? void 0 : _b.call(_a, "input[name=\"string-" + string + "\"][value=\"" + newFret + "\"]")) === null || _c === void 0 ? void 0 : _c.focus();
        (_d = props.onChange) === null || _d === void 0 ? void 0 : _d.call(props, set(strings, string, newFret));
    };
    useLayoutEffect(function () {
        var fretsNode = ref.current;
        if (center && fretsNode) {
            var children = fretsNode.querySelectorAll('.fret');
            var pressedFrets = strings.filter(function (fret) { return fret > 0; });
            var minFret = Math.min.apply(Math, pressedFrets);
            var maxFret = Math.max.apply(Math, pressedFrets);
            var toFret = minFret + Math.floor((maxFret - minFret) / 2);
            var fretNode = children[toFret];
            if (fretNode) {
                fretsNode.scrollLeft =
                    fretNode.offsetLeft -
                        fretsNode.offsetWidth / 2 +
                        fretNode.offsetWidth / 2;
            }
        }
    }, [ref, strings, center, lefty]);
    return (jsx("div", { ref: ref, css: styles, className: classNames('guitar', { lefty: lefty }, props.className) },
        jsx("div", { className: "sr-only" },
            "This is a guitar with ",
            strings.length,
            " strings and ",
            frets.amount,
            " frets, starting from ",
            frets.from,
            ".",
            props.onChange && jsx("span", null, "You can tab between strings and frets."),
            props.onPlay && (jsx("span", null, "When a specific string is focused you can play it by pressing 'p'."))),
        jsx("div", { className: "sr-only", role: "status" },
            "Current fretting: ",
            strings.join(', '),
            "."),
        jsx("div", { className: "strings" },
            jsx(Frets, { className: "fretboard", frets: frets }, theme.fret.marker
                ? function (fret) { var _a, _b; return jsx("div", { className: "marker" }, (_b = (_a = theme.fret).marker) === null || _b === void 0 ? void 0 : _b.call(_a, fret)); }
                : undefined),
            strings.map(function (currentFret, string) { return (jsx(Frets, { key: string, className: "string", currentFret: currentFret, frets: frets, onMouseEnter: function () { var _a; return playOnHover && ((_a = props.onPlay) === null || _a === void 0 ? void 0 : _a.call(props, string)); } }, function (fret) { return (jsx("label", null,
                jsx("span", { className: "sr-only", role: "status" },
                    "String ",
                    string + 1,
                    ", fret ",
                    fret,
                    ".",
                    ' ',
                    currentFret === -1 && 'This string is muted.'),
                fret >= 0 && (jsx("span", { className: "actual-string", style: {
                        opacity: currentFret === -1 ? 0.2 : 1,
                        borderBottom: "solid 0.2em " + color(theme.string.color(string)).darken(0.35),
                        backgroundColor: theme.string.color(string)
                    } })),
                jsx("input", { disabled: !props.onChange, type: "radio", name: "string-" + string, value: fret, checked: currentFret === fret, onChange: function (e) {
                        var _a;
                        (_a = props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, set(strings, string, fret));
                        e.target.focus();
                    }, onClick: function () { return fret === currentFret && releaseString(string); }, onKeyDown: function (e) {
                        var _a;
                        switch (e.keyCode) {
                            case 80:
                                (_a = props.onPlay) === null || _a === void 0 ? void 0 : _a.call(props, string);
                                break;
                            case 13:
                                releaseString(string);
                                e.preventDefault();
                        }
                    } }),
                jsx("span", { className: "finger" }, renderFinger === null || renderFinger === void 0 ? void 0 : renderFinger(string, fret === -1 ? 0 : fret)))); })); })),
        jsx(Frets, { className: "frame", frets: frets }, function (fret) { return (fret !== 0 ? jsx("span", { className: "counter" }, fret) : null); })));
}

export default Guitar;
export { getRenderFingerRelative, getRenderFingerSpn, spanishTheme, tunings, useSound };
//# sourceMappingURL=index.esm.js.map
